@@ -17,11 +17,13 @@ from alibi.api.defaults import (DEFAULT_DATA_KERNEL_SHAP,
                                 DEFAULT_META_KERNEL_SHAP,
                                 DEFAULT_META_TREE_SHAP)
 from alibi.api.interfaces import Explainer, Explanation, FitMixin
-from alibi.utils.distributed import DistributedExplainer
 from alibi.utils.wrappers import methdispatch
 
 if TYPE_CHECKING:
+    from alibi.utils.distributed import DistributedExplainer
     import catboost  # noqa F401
+else:
+    from alibi.utils import DistributedExplainer
 
 logger = logging.getLogger(__name__)
 
@@ -281,7 +283,8 @@ class KernelExplainerWrapper(KernelExplainer):
 
 class KernelShap(Explainer, FitMixin):
     # object that implements the explanation algorithm (set in fit)
-    _explainer: Union[KernelExplainerWrapper, DistributedExplainer]
+
+    _explainer: Union[KernelExplainerWrapper, 'DistributedExplainer']
 
     def __init__(self,
                  predictor: Callable[[np.ndarray], np.ndarray],
@@ -819,7 +822,7 @@ class KernelShap(Explainer, FitMixin):
             For more details, please see the shap library `documentation`_ .
 
                 .. _documentation:
-                   https://shap.readthedocs.io/en/latest/.
+                   https://shap.readthedocs.io/en/stable/.
 
         Returns
         -------
@@ -828,7 +831,7 @@ class KernelShap(Explainer, FitMixin):
             field containing additional data. See usage at `KernelSHAP examples`_ for details.
 
             .. _KernelSHAP examples:
-               https://docs.seldon.io/projects/alibi/en/latest/methods/KernelSHAP.html
+               https://docs.seldon.io/projects/alibi/en/stable/methods/KernelSHAP.html
         """  # noqa W605
 
         if not self._fitted:
@@ -899,7 +902,7 @@ class KernelShap(Explainer, FitMixin):
             field containing additional data. See usage at `KernelSHAP examples`_ for details.
 
             .. _KernelSHAP examples:
-               https://docs.seldon.io/projects/alibi/en/latest/methods/KernelSHAP.html
+               https://docs.seldon.io/projects/alibi/en/stable/methods/KernelSHAP.html
 
         """
 
@@ -1284,7 +1287,7 @@ class TreeShap(Explainer, FitMixin):
             `meta` field containing additional data. See usage at `TreeSHAP examples`_ for details.
 
             .. _TreeSHAP examples:
-               https://docs.seldon.io/projects/alibi/en/latest/methods/TreeSHAP.html
+               https://docs.seldon.io/projects/alibi/en/stable/methods/TreeSHAP.html
 
         """  # noqa: E501
 
@@ -1485,7 +1488,7 @@ class TreeShap(Explainer, FitMixin):
             `meta` field containing additional data. See usage at `TreeSHAP examples`_ for details.
 
             .. _TreeSHAP examples:
-               https://docs.seldon.io/projects/alibi/en/latest/methods/TreeSHAP.html
+               https://docs.seldon.io/projects/alibi/en/stable/methods/TreeSHAP.html
 
         """
 
