@@ -10,7 +10,9 @@ from alibi.api.interfaces import Explanation
 from alibi.explainers import IntegratedGradients
 from alibi.explainers.integrated_gradients import (_get_target_from_target_fn,
                                                    _run_forward_from_layer,
-                                                   _run_forward_to_layer)
+                                                   _run_forward_to_layer,
+                                                   _check_target,
+                                                   _select_target)
 
 # generate some dummy data
 N = 100
@@ -650,6 +652,65 @@ def test_run_forward_from_layer(layer_nb,
     assert np.allclose(preds_from_layer, expected_values)
 
 
+TARGETS_ARGS = [{'output_shape': (None, 2), 'target': np.array([0, 2]), 'nb_samples': 2, 'bad_target': True},
+                {'output_shape': (None,), 'target': np.array([0]), 'nb_samples': 2, 'bad_target': True},
+                {'output_shape': (None, 3), 'target': np.array([0, 0, 0]), 'nb_samples': 2, 'bad_target': True},
+                {'output_shape': (None, 3), 'target': np.array([-123, -123]), 'nb_samples': 2, 'bad_target': True},
+                {'output_shape': (None, 3), 'target': np.array([99, 99, 99]), 'nb_samples': 2, 'bad_target': True},
+                {'output_shape': (None, 3), 'target': 999, 'nb_samples': 2, 'bad_target': True},
+                {'output_shape': (None, 2), 'target': np.array([[0, 2]]), 'nb_samples': 2, 'bad_target': True},
+                {'output_shape': (None, 4, 4, 3), 'target': np.array([[0, 0], [0, 0]]),
+                 'nb_samples': 2, 'bad_target': True},
+                {'output_shape': (None, 4, 4, 3), 'target': np.array([[0, 0, 3], [0, 0, 0]]),
+                 'nb_samples': 2, 'bad_target': True},
+                {'output_shape': (None, 4, 4, 3), 'target': np.array([[0, 4, 0], [0, 0, 0]]),
+                 'nb_samples': 2, 'bad_target': True},
+                {'output_shape': (None, 4, 4, 3), 'target': np.array([[99, 99, 99], [0, 0, 0]]),
+                 'nb_samples': 2, 'bad_target': True},
+                {'output_shape': (None, 2), 'target': np.array([0, 1]), 'nb_samples': 2, 'bad_target': False},
+                {'output_shape': (None,), 'target': np.array([0, 1]), 'nb_samples': 2, 'bad_target': False},
+                {'output_shape': (None, 3), 'target': np.array([0, 2]), 'nb_samples': 2, 'bad_target': False},
+                {'output_shape': (None, 4, 4, 3), 'target': np.array([[0, 0, 0], [3, 3, 2]]),
+                'nb_samples': 2, 'bad_target': False}]
+
+SELECT_ARGS = [{'preds': np.array([[0.0, 0.1],
+                                   [1.0, 1.1]]),
+                'target': np.array([0,
+                                    0]),
+                'expected': np.array([0.0,
+                                      1.0])},
+               {'preds': np.array([[[0.0, 0.1], [1.0, 1.1]],
+                                   [[2.0, 2.1], [3.0, 4.1]]]),
+                'target': np.array([[0, 0],
+                                   [0, 0]]),
+                'expected': np.array([[0.0],
+                                      [2.0]])}]
+
+
+@pytest.mark.parametrize('args', TARGETS_ARGS)
+def test_check_target(args):
+    output_shape = args['output_shape']
+    target = args['target']
+    nb_samples = args['nb_samples']
+    bad_target = args['bad_target']
+
+    if bad_target:
+        with pytest.raises((ValueError, AttributeError)):
+            _check_target(output_shape, target, nb_samples)
+    else:
+        _check_target(output_shape, target, nb_samples)
+
+
+@pytest.mark.parametrize('args', SELECT_ARGS)
+def test_select_target(args):
+    preds = tf.convert_to_tensor(args['preds'])
+    target = tf.convert_to_tensor(args['target'])
+    expected = args['expected']
+
+    selected = _select_target(preds, target)
+    assert (expected == selected.numpy()).all()
+
+
 #####################################################################################################################
 
 
